subroutine RZstorage(init)

!=======================================================================
! Calculates the available root zone water storage capacity (model variable SFC)
! as a function of vegetation and long-term climate indicators.
!
! References:
! Guswa, A.J. (2008): The influence of climate on root depth: A carbon
! cost-benefit analysis. Water Resources Research, 44, W02427.
! DOI: 10.1029/2007WR006384
!
! Guswa, A.J. (2010): Effect of plant uptake strategy on the water−optimal root depth.
! Water Resources Research, 46, W09601.
! DOI: 10.1029/2010WR009122
!
! Speich, M.J.R., Lischke, H., and Zappa, M. (2018): Testing an optimality-based model
! of rooting zone water storage capacity in temperate forests. HESS, 22, 4097-4124.
! DOI: 10.5194/hess-22-4097-2018
!
! Called once a simulation year, after TreeMig update. Called from PREVAHO
!
!
!
! Implementation: Matthias Speich, Feb 2017
!
! New version - August 2018. Corresponds to what is described in the final version
! of Speich et al. (2018)
!
!=======================================================================

! MS, Dec. 2017: very high values for Zgrass when there is little forest cover. Occurs
! e.g. in Val d'Anniviers case study, at lower elevations. Possible explanation:
! Potential transpiration of "grass" is estimated as potential soil evaporation
! -> does not account for understory interception. This causes the wetness index to
! be close to 1 (in that region), and gives greater values than what we would get
! with the model for trees (Zfor). Possibly reduce PES (e.g. by a  factor 0.75) to
! account for understory interception?
!
!
! New version, August 2018:
! - Takes in annual potential evaporation (PE) instead of potential transpiration (Tpot) and
!   potential soil evaporation (pes). PE is dependent on climate only, and the effect of forest
!   properties is included by directly accounting for LAI here. This choice is based on the
!   suggestions of Andrew Guswa during review of the HESS paper, and ensures that the wetness
!   index is representative of climate. Also, this change allows the model to better represent the
!   effect of a rapid change in above-ground forest properties.


  use forhycsglobals, only: gsLength, gsPrec, gsPe, gsPrecDays, gsTemp, sfcMax, biomass2d, dstr30, dosfc, perCC, &
      TMtoPR, PRtoTM, fhla
  use prevahglobals, only : landin, ncol, nrow, rnodat, sfcG, whcSF
  use MPIvars, only : firstRow, lastRow
  use All_par, only: spec, stockability
  use auxroots

  implicit none

  logical, intent(in) :: init       ! If TRUE, set a standard SFC (initialization phase)
  integer :: xcoor, ycoor           ! Indices to loop over cells
  integer :: land                   ! Land cover type of current cell
  integer :: i                      ! Loop index
  real :: fgs                       ! Length of growing season (fraction of a year)
  real :: prec                      ! Precipitation (including snowmelt) during GS [mm]
  !real :: tpot                      ! Potential transpiration during GS [mm]
  !real :: pes                       ! Potential understory transpiration during GS [mm]
  real :: pe                        ! Potential evaporation during GS [mm]
  real :: lai                       ! Forest leaf area index in each cell
  real,parameter :: kl = 0.5        ! Canopy light extinction coefficient
  real :: tfrac                     ! Fraction of Tpot/ (Tpot + Pes)
  real :: pfreq                     ! Frequency of rainy days during GS
  real :: avdepth                   ! Average precipitation on a rainy day
  real :: whc                       ! Soil water holding capacity, as a fraction of soil volume
  real :: temp
  real :: rresp                     ! Root respiration rate
  real :: rrespGrass
  !real :: gamma20                   ! Root respiration rate at 20°C
  real :: rld                       ! Root length density
  real :: srl                       ! Specific root length
  !real :: wue                       ! Water-use efficiency
  real :: sumbio                    ! Total (above-ground) tree biomass
  real :: wetness                   ! Wetness index (=Prec/Tpot)
  real :: uwet                      ! Wetness index for understory
  real :: a,x                       ! Helper variables in the Guswa equations
  real :: zr                        ! Rooting depth
  real :: zgrass                    ! Rooting depth for grass/understory
  real :: zFor                      ! Rooting depth for forest fraction
  real :: sfc                       ! Effective plant-available root zone storage capacity
  real :: drstr                     ! Drought stress index (average value of the last 30 years)
  real :: reldstr                   ! For each species, current drought stress relative to maximum tolerated drought stress
  real :: swuedr                    ! Species-specific WUE including drought effects
  real :: fcc                       ! Fractional canopy cover
  integer :: j                      ! Loop index

  ! Just for testing
  real, dimension(30) :: fgsArr
  real :: fgsSum, fgsCount
  real :: zrbck

! Species-specific parameters

  ! Water use efficiency (WUE) [g C g-1 H2O]
  real, parameter :: swue(30) = (/0.0025,0.0015,0.002,0.002,0.0015,0.0015,0.0025,0.002,0.002,0.002,&
  0.002,0.0015,0.002,0.0015,0.0025,0.002,0.002,0.0025,0.002,0.002,&
  0.0015,0.0015,0.0015,0.0015,0.002,0.0015,0.0015,0.002,0.0025,0.0025/)

  ! Specific root length (SRL) [cm g-1]
  real, parameter :: ssrl(30) = (/3000,4000,3000,3000,3000,3000,3000,6000,6000,6000,&
  6000,6000,6000,6000,6000,6000,6000,6000,6000,6000,&
  6000,6000,6000,6000,6000,6000,6000,6000,6000,6000/)

  ! Root length density (RLD) [cm roots cm-3 soil]
  ! real, parameter :: srld(30) = (/0.02,0.024,0.02,0.02,0.02,0.02,0.02,0.03,0.03,0.03,&
  ! 0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,&
  ! 0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03/)
  real, parameter :: srld(30) = (/0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.4,0.4,0.4, &
  0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4, &
  0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4/)

  ! Root respiration rate at 20°C [g C g-1 roots day-1]
  !real, parameter :: sgamma20(30) = (/0.002,0.009,0.0055,0.0055,0.009,0.009,0.002,0.0055,0.0055,0.0055,&
 ! 0.0055,0.009,0.0055,0.009,0.002,0.0055,0.0055,0.002,0.0055,0.0055,&
 ! 0.009,0.009,0.009,0.009,0.0055,0.009,0.009,0.0055,0.002,0.002/)

  ! Maximum value for WUE, used for increasing WUE as a function of drought stress
  !real, parameter :: swueMax = 0.0025

  real, parameter :: q10 = 2.0  ! Q10 parameter for root respiration

  real, parameter :: gamma20 = 0.5
  real, parameter :: wue = 0.33

  real, parameter :: g20grass = 0.5     ! Root respiration rate - grass
  real, parameter :: rldGrass = 0.1       ! Root length density - grass
  real, parameter :: srlGrass = 1500      ! Specific root length - grass
  real, parameter :: wueGrass = 0.22      ! Water use efficiency - grass


  real(kind=8) zn, vpar, porp, oldporp, diff, ze
  integer :: zloop
  integer :: ifault
  real, dimension(30) :: fproj  ! Projection factor

!=======================================================================

 data (fproj(j),j=1,30) /0.8,0.8,0.8,0.8,0.8,0.8,0.8,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0/


  do ycoor = lastRow, firstRow, -1
  do xcoor = 1, ncol

  ! For now: forests/shrubs only, other land cover types have standard SFC
  land = landin(xcoor, ycoor)
  !if((land==3).or.(land==4).or.(land==5).or.(land==8)) then
  if(stockability(ycoor,xcoor) > 0.0) then

  if(dosfc == 1) then

  !if(init) then
  ! If coupled-spin-up is done: standard SFC value during the first 5 simulation years
 ! sfcG(xcoor,ycoor) = 30.
 ! else

! Mean drought stress, excluding those years for which there is no value (at the beginning of simulation)
    drstr = sum(dstr30(:,xcoor,ycoor),mask=dstr30(:,xcoor,ycoor).ne.rnodat) / count(dstr30(:,xcoor,ycoor).ne.rnodat)

  ! (Bio-)Climatic indicators - mean of all values of the last 30 years
  ! During spin-up, not all years are filled in the grids. Therefore, it is necessary to exclude the RNODATs

!    fgsArr = gsLength(:,xcoor,ycoor)

    fgs = sum(gsLength(:,xcoor,ycoor),mask=gsLength(:,xcoor,ycoor).ne.rnodat) / max(1,count(gsLength(:,xcoor,ycoor).ne.rnodat))
    pe = sum(gsPe(:,xcoor,ycoor),mask=gsPe(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsPe(:,xcoor,ycoor).ne.rnodat))
    prec = sum(gsPrec(:,xcoor,ycoor),mask=gsPrec(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsPrec(:,xcoor,ycoor).ne.rnodat))
    pfreq = (sum(gsPrecDays(:,xcoor,ycoor),mask=gsPrecDays(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsPrecDays(:,xcoor,ycoor).ne.rnodat))) / fgs
    avdepth = prec / (sum(gsPrecDays(:,xcoor,ycoor),mask=gsPrecDays(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsPrecDays(:,xcoor,ycoor).ne.rnodat)))
    temp = sum(gsTemp(:,xcoor,ycoor),mask=gsTemp(:,xcoor,ycoor).ne.rnodat)/max(1,count(gsTemp(:,xcoor,ycoor).ne.rnodat)) / fgs
    !fcc = perCC(xcoor,ycoor)
    lai = sum(fhla(xcoor,ycoor,1:30) * fproj) / 833.0
    lai = max(0.2, lai)

! Zero EP will cause NaNs
    if(pe.lt.0.1) pe = 0.1
    if(fgs==0.0) then
! Should occur only at the very beginning of simulation -
! Dummy values for variables calculated over the growing season
      fgs = 0.5
      prec = 2
      avdepth = 15
      temp = 5.0
    end if

  ! Annual to daily mean values
    pe = (pe / fgs) * 0.75    ! Reduced to account for understory interception
    prec = prec/fgs
    fgs = fgs/365.0  ! FGS as a fraction of a year
    wetness = prec / pe




  ! Vegetation parameters: average of the species-specific parameters, weighted by
  ! (above-ground) biomass

    sumbio = sum(biomass2d(xcoor,ycoor,1:30))

    rresp = 0.0
    !gamma20 = 0.0
    rld = 0.0
    srl = 0.0
    !wue = 0.0

    do i = 1,30
      rld = rld + (srld(i)*(biomass2d(xcoor,ycoor,i)/sumbio))
      srl = srl + (ssrl(i)*(biomass2d(xcoor,ycoor,i)/sumbio))
    end do

    ! Default values for RLD and SRL if there is no biomass yet (at the beginning of simulation,
    ! or when a cell becomes forested.) Otherwise these values will be NaN, leading to Inf root length
    if((sumbio==0.0).or.(rld.ne.rld).or.(srl.ne.srl)) then
      rld = 0.3
      srl = 4000
    end if


    ! Root respiration as a function of temperature
    rresp = gamma20 * q10**((temp-20)/10)

    ! Soil WHC
    whc = sfcMax(xcoor,ycoor)


  ! Rooting depth calculation for grass fraction (eqs. 16-18 and appendix in Guswa 2008)
    rrespGrass = g20Grass * q10**((temp-20)/10)

    a = (rrespGrass*rldGrass)/(srlGrass*wueGrass*pe)
    if(wetness.ge.1.0) then
      if((xcoor==56).and.(ycoor==97)) print*, "Wetness >= 1.0"
      x = wetness*((1+(whc/avdepth)*(((1-wetness)**2)/(2*a))) - sqrt((whc/avdepth)*(((1-wetness)**2)/(a)) &
       + (((whc/avdepth)*(((1-wetness)**2)/(2*a)))**2)))
       if(x.lt.0.00001) x = 0.00001
    else
    if((xcoor==56).and.(ycoor==97)) print*, "Wetness < 1.0"
      x = wetness*((1+(whc/avdepth)*(((1-wetness)**2)/(2*a))) + sqrt((whc/avdepth)*(((1-wetness)**2)/(a)) &
       + (((whc/avdepth)*(((1-wetness)**2)/(2*a)))**2)))

    end if

   zGrass = avdepth/(whc*(1-wetness)) * log(x)  ! Effective root depth, in mm


   ! New rooting depth calculation for forested fraction (Guswa 2010)

   ! Old Version - GIVES WRONG RESULTS!
   ! This is the version using a numerical approximation of Porporato's model, as described in the
   ! HESS-D paper. However, the rooting depth model is very sensitive to small differences in the
   ! derivative of the transpiration function. Therefore, this version may give results that
   ! greatly differ from the original model
!     vmcoeff = 1-exp(kv1*(wetness**kv2))
!     mcoeff = km1 * (1/wetness) * (1/(km2*sqrt(2*pi))) * exp(-((log(wetness)-km3)**2)/(2*(km2**2)))
!     zFor = (sqrt(fgs*tpot*(whc/avdepth)*wue*srl*vmcoeff*mcoeff/(rresp*rld)) - mcoeff) * (avdepth/whc)

    ! New version - THIS IS THE CORRECT VERSION
    vpar = (rresp*rld) / (srl * wue * fgs)

    porp = -999.

    do zloop=300, 10000

      oldporp = porp
      ze = real(zloop)/10.
      zn = ze * (whc/avdepth)
      porp = pe * wetness - ((exp(-zn) * zn**(wetness*zn-1))/(gamain(zn,(wetness*zn),ifault)*gamma(wetness*zn)))
      diff = (porp-oldporp) * 10.0
      if(diff.le.vpar) exit
    end do

    zFor = ze

    ! Avoid NaNs (may otherwise occur if there is no tree biomass)
    if(zFor.ne.zFor) zFor = 0.0


    ! Average of grass and forest fractions
    zr = zFor * (1-exp(-kl*lai)) + zGrass * exp(-kl*lai)

    ! For UNCOUPLED or ONE-WAY COUPLED simulations: no dynamic rooting depth,
    ! Zr is always set to 1 m.
    if(TMtoPR.eq.0.or.PRtoTM.eq.0) zr = 1000.

    ! Model assumption: soil may not be thicker than 1m. If zr > 1000mm, set to 1000mm
    zrbck = zr
    if(zr.gt.1000) zr = 1000.

    ! Effective plant-available water holding capacity [mm] is assumed to be the product of
    ! effective root depth (zr) and soil water holding capacity
    sfc = zr * whc

    ! Potentially forested cells where forest is not (yet) growing well: take standard LC-dependent
    ! rooting depth if it is greater than the simulated rooting depth
    if((land==21).or.(land==23).or.(land==24).or.(land==25).or.(land==26)) then
      sfc = max(sfc, whc*whcSF(land))
    end if

    ! Update global SFC grid
    sfcG(xcoor,ycoor) = max(30.,sfc)
    

  !end if  ! Init

  else ! Control parameter dosfc == 0
  sfcG(xcoor,ycoor) = 1000 * sfcMax(xcoor,ycoor)

  end if  ! dosfc

  end if  ! (Land) Stockability

  end do
  end do


end subroutine RZstorage
