! Subroutine of the model FORHYCS

! Adjusts the maximum height of a tree species based on prevailing drought conditions
! (assumption: hydraulic limitation of vertical tree growth).
!
! Matthias Speich, WSL,  April 2017

subroutine adjhmax

  use forhycsglobals, only: dstr30, gdd30, curhcmax, PRtoTM,nspc_g, dohmax
  use prevahglobals, only: rnodat, landin, ncol
  use MPIvars, only: firstRow, lastRow, rank  ! Rank for debug only
  use All_par, only: stockability, spec, htmax, maxhc, birht

  implicit none

  ! Height reduction parameter (fraction of species-specific Hmax), from Rasche et al. (2012)
  real, parameter, dimension(30) :: kRedMAx = (/0.44, 0.38, 0.34, 0.38, 0.38, 0.38, 0.38, &
  0.8, 0.58, 0.5, 0.68, 0.68, 0.68, 0.63, 0.56, 0.4, 0.68, 0.43, 0.4, 0.78, 0.72, 0.43, 0.42, 0.42, &
  0.68, 0.68, 0.68, 0.4, 0.4, 0.4/)
  integer :: xcoor, ycoor, s
  real :: drstr, gdd, kdrstr, kddmin, kd75, relh, relh_d, relh_g, newHmax
  integer :: newhc
  real :: debugstr, debugrelh, debughmax
  integer :: debugnewhc
  logical :: doAdjHmax

  real :: rmax

  !========================================

  if(dohmax==1) then
    doAdjHmax = .TRUE.
  else
    doAdjHmax = .FALSE.
  end if

  ! Degree-day sums: as TreeMig uses a function that asymptotically approaches 100% growth with
  ! increasing GDD, it is assumed here that maximum height is not limited by GDD sums above the
  ! level of 75% growth (given by the 2 input parameters kDDMin and "kDDMax" (delta75))

  ! Loop over grid cells
  do ycoor = lastRow, firstRow, -1
  do xcoor = 1,ncol
    if(stockability(ycoor,xcoor).lt.1) cycle  ! Applies to forested cells only

    ! Mean drought stress and GDD, excluding those years for which there is no value (at the beginning of simulation)
    drstr = sum(dstr30(:,xcoor,ycoor),mask=dstr30(:,xcoor,ycoor).ne.rnodat) / count(dstr30(:,xcoor,ycoor).ne.rnodat)
    gdd = sum(gdd30(:,xcoor,ycoor),mask=gdd30(:,xcoor,ycoor).ne.rnodat) / count(gdd30(:,xcoor,ycoor).ne.rnodat)

    if((xcoor==40).and.(ycoor==36)) then
      print*, "dstr30", dstr30(:,xcoor,ycoor)
    end if

    do s = 1,nspc_g

      if(doAdjHmax) then
      ! Get drought tolerance parameter of current species. Depending on whether this is an uncoupled TreeMig run or
      ! a coupled FORHYCS run, the parameters refer to the abiotic evapotranspiration index (Bugmann 1994, Fischlin et al.
      ! 1995) or to the transpiration index (Lischke and Zierl 2002). As of now (March 2017), transpiration index
      ! tolerance parameters are hard-coded in the subroutine TreeMig, where spc(i)%kdrt is overwritten.

      kdrstr = spec(s)%kdrt
      kd75 = spec(s)%kddmax
      kddmin = spec(s)%kddmin
      ! Maximum height for current species limited by drought, as a fraction of global Hmax(s)

      rmax = kredMax(s)

      ! Test: use non-linear functions (same as for growth reduction) instead of the linear
      ! functions of Rasche et al.
      relh_d = max(rmax,((1.0-(drstr/kdrstr))**0.5))
      relh_g = max(rmax,1.0-exp((1.386/kd75)*(kddmin-gdd)))
      ! Minimum principle (as in Rasche et al.)
      relh = min(relh_d,relh_g)

      ! Absolute value
      newHmax = spec(s)%khmax * relh
      ! Convert to height class - copied from GetSpc
      newhc = MIN (FLOOR (REAL (maxhc - 1) * &
                (newHmax - birht) / (REAL (htmax) - birht) + 1.0), maxhc)
      curhcmax(s,xcoor,ycoor) = newhc

      if((xcoor==40).and.(ycoor==36).and.(s==1)) then
        debugstr = drstr
        debugrelh = relh
        debughmax = newHmax
        debugnewhc = newhc
      end if


      else  ! No drought-dependent adjustment of Hmax
        curhcmax(s,xcoor,ycoor) = spec(s)%maxhtc

      end if
    end do

  end do
  end do

end subroutine adjhmax
