! Jarvis-type scheme for calculating stomatal resistance.

! Matthias Speich, WSL

!-------------------------------------------------------------------------------
! References:
!
! Jarvis, P.G. (1976): The interpretation of the variations in leaf water potential
! and stomatal conductance found in canopies in the field. Phil. Trans. R. Soc. Lond. B. 273, 593-610.
!
! Stewart, J.B. (1988): Modelling surface conductance of pine forest. Agric. For. Meteorol. 43:19-35
!
! ASCE Hydrology Handbook (1996), Chapter 4: Evaporation and Transpiration
!
! Medlyn, B. et al. (2001): Stomatal conductance of forest species after long-term exposure to elevated
! CO2 concentration: a synthesis. New Phytologist 149(2):247-264
!
! Medlyn, B. et al. (1999?): Meta-Analysis of Model Parameters. Chapter 4 in: Predicted Impacts of Rising
! Carbon Dioxide and Temperature on Forests in Europe at Stand Scale. ECOCRAFT Final Report
!-------------------------------------------------------------------------------


subroutine erstomJarvis(rlai,rsmin,rad,land,sm,sfc,vp,vpSat,tAir,ca,rs,pa,xcoor,ycoor)

implicit none


! Passed variables
  real, intent(in) :: rlai    ! Effective LAI
  real, intent(in) :: rsmin   ! Minimum stomatal resistance
  real, intent(in) :: rad     ! Global radiation
  integer, intent(in) :: land ! Land cover class
  real, intent(in) :: sm      ! Soil moisture content
  real, intent(in) :: sfc     ! Size of plant available soil water reservoir
  real, intent(in) :: vp      ! Water vapor pressure [hPa]
  real, intent(in) :: vpSat   ! Saturated vapor pressure [hPa]
  real, intent(in) :: tAir    ! Air temperature
  real, intent(in) :: ca      ! CO2 concentration in air [ppm]
  real, intent(out) :: rs  ! Stomatal resistance [m s-1]
  integer, intent(in) :: pa  ! 1 if potential transpiration is to be estimated, 2 if actual transpiration
  integer, intent(in) :: xcoor, ycoor  ! DEBUG ONLY

! Internal variables
  real :: jR     ! Parameter for radiation response function, set to 100 [W m-2]
  real :: vpd     ! Vapor pressure deficit
  real :: f1, f2, f3, f4, f5 ! Results of the 4 reduction functions (0.0 ... 1.0)
  real, parameter :: rsmax = 5000.0  ! Maximum stomatal resistance
  !real, parameter :: jv1 = 0.05 ! Parameter for exponential part of VPD response function [hPa-1]
  !real, parameter :: jv2 = 15. ! Threshold VPD value [hPa]
  real, parameter :: jvpd = -0.08  ! VPD response parameter for negative exponential response
  real, parameter :: jsm = 6.7 ! Empirical parameter for soil moisture function
  real :: jC  ! Parameter for CO2 response function
  real, parameter :: tl = 0.        ! Minimum, optimal and maximum temperature for PS
  real, parameter :: topt = 18.
  real, parameter :: th = 40.
  real :: a  ! Helper variable for temperature response function



! 1st reduction function: global radiation
  jR = 100.
  f1 = (1000/(1000+jr)) * ((rad+jr)/rad)


! 2nd reduction function: Air temperature
  if(tAir < tl) then
    f2 = 20  ! ~ the value it would take with tAir==tl in Stewart parameterization (tl = 0)
  else
    a = (th-topt)/(topt-tl)
    f2 = 1/(((tAir-tl)*((th-tAir)**a))/((topt-tl)*((th-topt)**a)))
  end if

! 3rd reduction function: vapor pressure deficit
  if (pa==1) then
    f3 = 1
  else
!     vpd = vpSat-vp
!     if(vpd < jv2) then
!       f3 = 1/(1-(vpd*jv1))
!     else
!       f3 = 1/(1-(jv2*jv1))
!     end if
    vpd = vpSat-vp
    f3 = 1/(exp(jvpd*vpd))
  end if


! 4th reduction function: soil moisture
  if(pa==1) then
    f4 = 1
  else
    f4 = 1/(1-exp(-jsm*(sm/sfc)))
  end if


! 5th reduction function: CO2 concentration.
  select case(land)
    case(3)
      jC = 0.1
    case(4)
      jC = 0.25
    case default
      jC = 0.18
  end select
  f5 = max(1.0, 1/(1-jc*((min(ca,700.)/350.)-1)))


! Calculating stomatal resistance:
  rs = (rsMin/rlai) * (f1*f2*f3*f4*f5)
  !rs = (rsMin) * (f1/(f2*f3*f4*f5))   ! With MORECS parameters: rsMin is already reduced according to LAI
  rs = max(rs,rsmin/rlai)
  rs = min(rs,rsmax)



end subroutine erstomJarvis
